package com.alterdekim.game.service;

import com.alterdekim.game.component.game.response.location.LocationObject;
import com.alterdekim.game.entity.Location;
import com.alterdekim.game.entity.LocationObjectInstance;
import com.alterdekim.game.repository.LocationRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Optional;

@Service
public class LocationService {

    @Autowired
    private LocationRepository repository;

    public List<LocationObjectInstance> getAllLocationObjects() {
        return this.repository.findAllLocationObjects();
    }

    public LocationObject getDefaultLocation() {
        Optional<Location> l = this.repository.findDefaultLocation();
        if( l.isPresent() ) {
            Location r = l.get();
            return new LocationObject(r.getLocationId(), false, false, false, true, r.getMediaResourceId(), r.getX(), r.getY(), r.getName(), this.repository.findObjectsForLocationById(r.getLocationId()));
        }
        return new LocationObject();
    }

    public LocationObject getLocationById(Long locationId) {
        Optional<Location> l = this.repository.findLocationById(locationId);
        if( l.isPresent() ) {
            Location r = l.get();
            return new LocationObject(r.getLocationId(), false, false, false, true, r.getMediaResourceId(), r.getX(), r.getY(), r.getName(), this.repository.findObjectsForLocationById(locationId));
        }
        return getDefaultLocation();
    }

    public void addLocation(Location location) {
        this.repository.save(location);
    }

    public List<Location> getAll() {
        return repository.findAll();
    }

    public void removeById(Long id) {
        this.repository.deleteById(id);
    }

    public void removeObjectById(Long id) {
        this.repository.deleteObjectById(id);
    }
}
